@@ -7,7 +7,7 @@ from distutils.util import strtobool
 from cerberus_kind.utils import parse_error, kind_schema
 from cerberus_document_editor import yaml_parser
 from .validator import Validator
-from .widget import Widget, FlatButton
+from .widget import Widget, ComboBox
 from .page import ListPage, PopupPage
 from .debug import log
 
@@ -15,7 +15,7 @@ def BOOLEAN(x):
     return bool(strtobool(x))
 
 # Helper functions
-def ellipsis(text, max_w=60, max_h=10):
+def ellipsis(text, max_w=60, max_h=0):
     def cols(rows):
         output = []
         for row in rows:
@@ -25,7 +25,7 @@ def ellipsis(text, max_w=60, max_h=10):
                 output.append(row)
         return output
     rows = text.strip('\n').split('\n')
-    if len(rows) > max_h:
+    if max_h != 0 and len(rows) > max_h:
         return '\n'.join(cols(rows[:max_h-1]+['...']))
     else:
         return '\n'.join(cols(rows))
@@ -42,6 +42,15 @@ def callback_generator(ctx, name, schema, doc):
         ctx.next(page)
     return callback
 
+def get_selector_info(document, schema):
+    allowed = [_.title() for _ in schema['selector']]
+    kind = document.get('kind', '').lower()
+    if not kind in schema['selector']:
+        allowed += [kind.title()]
+        kind = allowed[0].lower()
+    #schema = schema['selector'].get(kind)
+    return kind, allowed
+
 class EditorPage(ListPage):
     def __init__(self, name, schema, document, sub_page=False):
         super().__init__(name, sub_page=sub_page)
@@ -97,9 +106,9 @@ class EditorPage(ListPage):
                         self.warning('Already exist key.')
                     else:
                         body['document'][key] = None
+                        body['document'] = self.validator.normalized(body['document'], ordered=True)
                         self.json = body
                         self.modified()
-                        self.render()
             elif 'rename' in page.json:
                 if hasattr(self, '_last_key'):
                     last_key = getattr(self, '_last_key')
@@ -114,7 +123,6 @@ class EditorPage(ListPage):
                             )
                             self.json = body
                             self.modified()
-                            self.render()
             elif 'exit' in page.json:
                 key = page.json.get('exit')
                 if key.lower() == 'yes':
@@ -123,7 +131,6 @@ class EditorPage(ListPage):
                     self.hwnd.destroy(False)
                 elif key.lower() == 'cancel':
                     ...
-                self.render()
             else:
                 data = self.json
                 document = data['document']
@@ -136,7 +143,6 @@ class EditorPage(ListPage):
                 if self.json != data:
                     self.json = data
                     self.modified()
-                self.render()
 
     def on_change(self, widget, new_value):
         def casting(value):
@@ -181,13 +187,23 @@ class EditorPage(ListPage):
             if self.json['document'] != document:
                 self.json = {"document": document}
                 self.modified()
-            self.render()
+            
+            REFRESH_WIDGETS = [ComboBox]
+            if type(widget) in REFRESH_WIDGETS:
+                self.render()
 
             # Update indicator for focusing item.
             if '__root__' in schema:
                 schema = schema['__root__']
                 if schema.get('selector'):
-                    schema = schema['selector'][document['kind'].lower()]
+                    # allowed = [_.title() for _ in schema['selector']]
+                    # kind = document.get('kind', '').lower()
+                    # if not kind in schema['selector']:
+                    #     allowed += [kind.title()]
+                    #     kind = allowed[0].lower()
+                    # schema = schema['selector'].get(kind)
+                    kind, _ = get_selector_info(document, schema)
+                    schema = schema['selector'].get(kind)
                 elif schema.get('valuesrules'):
                     schema = {key: schema['valuesrules'] for key in document}
                 elif schema.get('oneof'):
@@ -216,11 +232,12 @@ class EditorPage(ListPage):
             if schema.get('selector'):
                 # Selector 일 경우
                 self._config['root_type'] = 'selector'
-                allowed = [_.title() for _ in schema['selector']]
-                kind = doc.get('kind', "").lower()
-                if not kind in schema['selector']:
-                    allowed += [kind.title()]
-                    kind = allowed[0].lower()
+                # allowed = [_.title() for _ in schema['selector']]
+                # kind = doc.get('kind', "").lower()
+                # if not kind in schema['selector']:
+                #     allowed += [kind.title()]
+                #     kind = allowed[0].lower()
+                kind, allowed = get_selector_info(doc, schema)
                 schema = schema['selector'].get(kind)
                 schema['kind'] = kind_schema(kind, allowed)
                 log("** KIND ", schema['kind'])
@@ -239,8 +256,8 @@ class EditorPage(ListPage):
                 if schema.get('type') == 'list':
                     self._config['root_type'] = 'list'
 
-        log('root schema:', list(self._config['root_schema'].keys()))
-        log('root type:', self._config['root_type'])
+            log('root schema:', list(self._config['root_schema'].keys()))
+            log('root type:', self._config['root_type'])
 
         # Prepare appendable items with hotkey
         if self.is_valuesrules:
